@@ -60,10 +60,12 @@ except:
 from . import braille
 from . import debug
 from . import event_manager
+from . import keybindings
 from . import logger
 from . import messages
 from . import notification_messages
 from . import orca_state
+from . import orca_platform
 from . import script_manager
 from . import settings
 from . import settings_manager
@@ -215,12 +217,17 @@ def _processKeyboardEvent(event):
     """
     global _orcaModifierPressed
 
-    # Weed out duplicate and otherwise bogus events.
     keyboardEvent = KeyboardEvent(event)
     debug.println(debug.LEVEL_FINE, keyboardEvent.toString())
-    if keyboardEvent.ignoreDueToTimestamp():
-        debug.println(debug.LEVEL_FINE, "IGNORING EVENT DUE TO TIMESTAMP")
-        return
+
+    # Weed out duplicate and otherwise bogus events.
+    # TODO - JD: Be sure these are the right values to return
+    if not keyboardEvent.timestamp:
+        debug.println(debug.LEVEL_FINE, "IGNORING EVENT: NO TIMESTAMP")
+        return False
+    if keyboardEvent == orca_state.lastInputEvent:
+        debug.println(debug.LEVEL_FINE, "IGNORING EVENT: DUPLICATE")
+        return False
 
     # Figure out what we've got.
     isOrcaModifier = keyboardEvent.isOrcaModifier()
@@ -228,10 +235,9 @@ def _processKeyboardEvent(event):
     if isOrcaModifier:
         _orcaModifierPressed = isPressedEvent
     if _orcaModifierPressed:
-        keyboardEvent.modifiers |= settings.ORCA_MODIFIER_MASK
+        keyboardEvent.modifiers |= keybindings.ORCA_MODIFIER_MASK
 
     # Update our state.
-    orca_state.lastInputEventTimestamp = event.timestamp
     orca_state.lastInputEvent = keyboardEvent
     if not keyboardEvent.isModifierKey():
         keyboardEvent.setClickCount()
@@ -239,12 +245,15 @@ def _processKeyboardEvent(event):
 
     # Echo it based on what it is and the user's settings.
     script = orca_state.activeScript
-    if script:
-        if isPressedEvent:
-            script.presentationInterrupt()
-        keyboardEvent.present()
-        if keyboardEvent.isModifierKey() and not isOrcaModifier:
-            return False
+    if not script:
+        debug.println(debug.LEVEL_FINE, "IGNORING EVENT DUE TO NO SCRIPT")
+        return False
+
+    if isPressedEvent:
+        script.presentationInterrupt()
+    script.presentKeyboardEvent(keyboardEvent)
+    if keyboardEvent.isModifierKey() and not isOrcaModifier:
+        return False
  
     # Special modes.
     if not isPressedEvent and keyboardEvent.event_string == "Escape":
@@ -258,7 +267,7 @@ def _processKeyboardEvent(event):
     if orca_state.capturingKeys:
         return False
     if notification_messages.listNotificationMessagesModeEnabled:
-        return notification_messages.listNotificationMessages(keyboardEvent)
+        return notification_messages.listNotificationMessages(script, keyboardEvent)
 
     # See if the event manager wants it (i.e. it is bound to a command.
     if _eventManager.processKeyboardEvent(keyboardEvent):
@@ -434,6 +443,8 @@ def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
         except:
             debug.printException(debug.LEVEL_SEVERE)
 
+    _settingsManager.loadAppSettings(script)
+
     if _settingsManager.getSetting('enableSpeech'):
         try:
             speech.init()
@@ -483,6 +494,27 @@ def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
 
     return True
 
+def _showPreferencesUI(script, prefs):
+    if orca_state.orcaOS:
+        orca_state.orcaOS.showGUI()
+        return
+
+    try:
+        module = importlib.import_module('.orca_gui_prefs', 'orca')
+    except:
+        debug.printException(debug.LEVEL_SEVERE)
+        return
+
+    uiFile = os.path.join(orca_platform.prefix,
+                          orca_platform.datadirname,
+                          orca_platform.package,
+                          "ui",
+                          "orca-setup.ui")
+
+    orca_state.orcaOS = module.OrcaSetupGUI(uiFile, "orcaSetupWindow", prefs)
+    orca_state.orcaOS.init(script)
+    orca_state.orcaOS.showGUI()
+
 def showAppPreferencesGUI(script=None, inputEvent=None):
     """Displays the user interace to configure the settings for a
     specific applications within Orca and set up those app-specific
@@ -491,11 +523,12 @@ def showAppPreferencesGUI(script=None, inputEvent=None):
     Returns True to indicate the input event has been consumed.
     """
 
-    try:
-        module = importlib.import_module('.app_gui_prefs', 'orca')
-        module.showPreferencesUI()
-    except:
-        debug.printException(debug.LEVEL_SEVERE)
+    prefs = {}
+    for key in settings.userCustomizableSettings:
+        prefs[key] = _settingsManager.getSetting(key)
+
+    script = script or orca_state.activeScript
+    _showPreferencesUI(script, prefs)
 
     return True
 
@@ -506,11 +539,9 @@ def showPreferencesGUI(script=None, inputEvent=None):
     Returns True to indicate the input event has been consumed.
     """
 
-    try:
-        module = importlib.import_module('.orca_gui_prefs', 'orca')
-        module.showPreferencesUI()
-    except:
-        debug.printException(debug.LEVEL_SEVERE)
+    prefs = _settingsManager.getGeneralSettings(_settingsManager.profile)
+    script = _scriptManager.getDefaultScript()
+    _showPreferencesUI(script, prefs)
 
     return True
 
@@ -593,7 +624,7 @@ def init(registry):
 
     return True
 
-def start(registry):
+def start(registry, cacheValues):
     """Starts Orca.
     """
 
@@ -611,11 +642,11 @@ def start(registry):
     if settings.timeoutCallback and (settings.timeoutTime > 0):
         signal.alarm(0)
 
-    if settings.cacheValues:
+    if cacheValues:
         pyatspi.setCacheLevel(pyatspi.CACHE_PROPERTIES)
 
     debug.println(debug.LEVEL_FINEST, 'INFO: Orca starting registry')
-    registry.start(gil=settings.useGILIdleHandler)
+    registry.start(gil=False)
 
 def die(exitCode=1):
     pid = os.getpid()
@@ -731,7 +762,7 @@ def abortOnSignal(signum, frame):
                   % signum)
     die(signum)
 
-def main():
+def main(cacheValues=True):
     """The main entry point for Orca.  The exit codes for Orca will
     loosely be based on signals, where the exit code will be the
     signal used to terminate Orca (if a signal was used).  Otherwise,
@@ -772,7 +803,7 @@ def main():
             setLocusOfFocus(None, window)
 
     try:
-        start(pyatspi.Registry) # waits until we stop the registry
+        start(pyatspi.Registry, cacheValues) # waits until we stop the registry
     except:
         die(EXIT_CODE_HANG)
     return 0
