@@ -710,15 +710,16 @@ class Utilities:
                              pyatspi.ROLE_LIST_ITEM,
                              pyatspi.ROLE_TREE_ITEM]
 
-        if role == pyatspi.ROLE_TABLE:
-            layoutOnly = attrs.get('layout-guess') == 'true'
-            if not layoutOnly:
-                try:
-                    table = obj.queryTable()
-                except:
+        if role == pyatspi.ROLE_TABLE and attrs.get('layout-guess') != 'true':
+            try:
+                table = obj.queryTable()
+            except:
+                layoutOnly = True
+            else:
+                if not (table.nRows and table.nColumns):
                     layoutOnly = True
-                else:
-                    layoutOnly = table.nRows <= 1 or table.nColumns <= 1
+                elif not (obj.name or self.displayedLabel(obj)):
+                    layoutOnly = not (table.getColumnHeader(0) or table.getRowHeader(0))
         elif role == pyatspi.ROLE_TABLE_CELL and obj.childCount:
             if obj[0].getRole() == pyatspi.ROLE_TABLE_CELL:
                 layoutOnly = True
@@ -2704,7 +2705,7 @@ class Utilities:
         # Given an broken table hierarchy, findDescendant can hang. And the
         # reason we're here in the first place is to work around the app or
         # toolkit killing accessibles. There's only so much we can do....
-        if root.getRole() == pyatspi.ROLE_TABLE:
+        if root.getRole() in [pyatspi.ROLE_TABLE, pyatspi.ROLE_EMBEDDED]:
             return None
 
         isSame = lambda x: x and self.isSameObject(
@@ -2753,7 +2754,9 @@ class Utilities:
 
         parent = self.getFunctionalParent(obj)
         siblings = self.getFunctionalChildren(parent)
-        siblings = list(filter(lambda x: not self.isLayoutOnly(x), siblings))
+        layoutRoles = [pyatspi.ROLE_SEPARATOR, pyatspi.ROLE_TEAROFF_MENU_ITEM]
+        isNotLayoutOnly = lambda x: x and not x.getRole() in layoutRoles
+        siblings = list(filter(isNotLayoutOnly, siblings))
         if not (siblings and obj in siblings):
             return -1, -1
 
